/****************************************************************************
** Meta object code from reading C++ file 'uniqueapphandler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kontactinterface-23.08.5/src/uniqueapphandler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uniqueapphandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KontactInterface__UniqueAppHandler_t {
    QByteArrayData data[9];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KontactInterface__UniqueAppHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KontactInterface__UniqueAppHandler_t qt_meta_stringdata_KontactInterface__UniqueAppHandler = {
    {
QT_MOC_LITERAL(0, 0, 34), // "KontactInterface::UniqueAppHa..."
QT_MOC_LITERAL(1, 35, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 51, 28), // "org.kde.PIMUniqueApplication"
QT_MOC_LITERAL(3, 80, 11), // "newInstance"
QT_MOC_LITERAL(4, 92, 0), // ""
QT_MOC_LITERAL(5, 93, 6), // "asn_id"
QT_MOC_LITERAL(6, 100, 4), // "args"
QT_MOC_LITERAL(7, 105, 16), // "workingDirectory"
QT_MOC_LITERAL(8, 122, 4) // "load"

    },
    "KontactInterface::UniqueAppHandler\0"
    "D-Bus Interface\0org.kde.PIMUniqueApplication\0"
    "newInstance\0\0asn_id\0args\0workingDirectory\0"
    "load"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KontactInterface__UniqueAppHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       2,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    3,   26,    4, 0x0a /* Public */,
       8,    0,   33,    4, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Int, QMetaType::QByteArray, QMetaType::QStringList, QMetaType::QString,    5,    6,    7,
    QMetaType::Bool,

       0        // eod
};

void KontactInterface::UniqueAppHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UniqueAppHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { int _r = _t->newInstance((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->load();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KontactInterface::UniqueAppHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KontactInterface__UniqueAppHandler.data,
    qt_meta_data_KontactInterface__UniqueAppHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KontactInterface::UniqueAppHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KontactInterface::UniqueAppHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KontactInterface__UniqueAppHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KontactInterface::UniqueAppHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_KontactInterface__UniqueAppWatcher_t {
    QByteArrayData data[6];
    char stringdata0[82];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KontactInterface__UniqueAppWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KontactInterface__UniqueAppWatcher_t qt_meta_stringdata_KontactInterface__UniqueAppWatcher = {
    {
QT_MOC_LITERAL(0, 0, 34), // "KontactInterface::UniqueAppWa..."
QT_MOC_LITERAL(1, 35, 22), // "slotApplicationRemoved"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 4), // "name"
QT_MOC_LITERAL(4, 64, 8), // "oldOwner"
QT_MOC_LITERAL(5, 73, 8) // "newOwner"

    },
    "KontactInterface::UniqueAppWatcher\0"
    "slotApplicationRemoved\0\0name\0oldOwner\0"
    "newOwner"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KontactInterface__UniqueAppWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    3,    4,    5,

       0        // eod
};

void KontactInterface::UniqueAppWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UniqueAppWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotApplicationRemoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KontactInterface::UniqueAppWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KontactInterface__UniqueAppWatcher.data,
    qt_meta_data_KontactInterface__UniqueAppWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KontactInterface::UniqueAppWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KontactInterface::UniqueAppWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KontactInterface__UniqueAppWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KontactInterface::UniqueAppWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
