// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kontactinterface_debug.h"


Q_LOGGING_CATEGORY(KONTACTINTERFACE_LOG, "org.kde.pim.kontactinterface", QtInfoMsg)

